/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/nirmata/kyverno/pkg/api/kyverno/v1"
	scheme "github.com/nirmata/kyverno/pkg/client/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// GenerateRequestsGetter has a method to return a GenerateRequestInterface.
// A group's client should implement this interface.
type GenerateRequestsGetter interface {
	GenerateRequests(namespace string) GenerateRequestInterface
}

// GenerateRequestInterface has methods to work with GenerateRequest resources.
type GenerateRequestInterface interface {
	Create(*v1.GenerateRequest) (*v1.GenerateRequest, error)
	Update(*v1.GenerateRequest) (*v1.GenerateRequest, error)
	UpdateStatus(*v1.GenerateRequest) (*v1.GenerateRequest, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.GenerateRequest, error)
	List(opts metav1.ListOptions) (*v1.GenerateRequestList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.GenerateRequest, err error)
	GenerateRequestExpansion
}

// generateRequests implements GenerateRequestInterface
type generateRequests struct {
	client rest.Interface
	ns     string
}

// newGenerateRequests returns a GenerateRequests
func newGenerateRequests(c *KyvernoV1Client, namespace string) *generateRequests {
	return &generateRequests{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the generateRequest, and returns the corresponding generateRequest object, and an error if there is any.
func (c *generateRequests) Get(name string, options metav1.GetOptions) (result *v1.GenerateRequest, err error) {
	result = &v1.GenerateRequest{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("generaterequests").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of GenerateRequests that match those selectors.
func (c *generateRequests) List(opts metav1.ListOptions) (result *v1.GenerateRequestList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.GenerateRequestList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("generaterequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested generateRequests.
func (c *generateRequests) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("generaterequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a generateRequest and creates it.  Returns the server's representation of the generateRequest, and an error, if there is any.
func (c *generateRequests) Create(generateRequest *v1.GenerateRequest) (result *v1.GenerateRequest, err error) {
	result = &v1.GenerateRequest{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("generaterequests").
		Body(generateRequest).
		Do().
		Into(result)
	return
}

// Update takes the representation of a generateRequest and updates it. Returns the server's representation of the generateRequest, and an error, if there is any.
func (c *generateRequests) Update(generateRequest *v1.GenerateRequest) (result *v1.GenerateRequest, err error) {
	result = &v1.GenerateRequest{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("generaterequests").
		Name(generateRequest.Name).
		Body(generateRequest).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *generateRequests) UpdateStatus(generateRequest *v1.GenerateRequest) (result *v1.GenerateRequest, err error) {
	result = &v1.GenerateRequest{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("generaterequests").
		Name(generateRequest.Name).
		SubResource("status").
		Body(generateRequest).
		Do().
		Into(result)
	return
}

// Delete takes name of the generateRequest and deletes it. Returns an error if one occurs.
func (c *generateRequests) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("generaterequests").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *generateRequests) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("generaterequests").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched generateRequest.
func (c *generateRequests) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.GenerateRequest, err error) {
	result = &v1.GenerateRequest{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("generaterequests").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
